package telecomnancy.directdealing.controller;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ListView;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TextField;
import lombok.Getter;
import telecomnancy.directdealing.App;
import telecomnancy.directdealing.database.DatabaseManager;
import telecomnancy.directdealing.model.User;

public class MessagerieController {
    private static String path = "src/main/resources/DirectDealing.sqlite";
    @Getter
    private static String dbPath = "jdbc:sqlite:" + path;
    @FXML 
    MenuItem deco;
    @FXML
    MenuItem infoflo;
    @FXML
    MenuItem quitter;
    @FXML
    MenuItem profil;
    @FXML
    MenuItem tdb;
    @FXML
    MenuItem offres;
    @FXML
    ListView<String> lvContacts;
    @FXML
    ListView<String> lvMessages;
    @FXML
    TextField message;
    @FXML
    Button envoyer;
    @FXML
    Menu florains;
    private App app;
    private User user;
    private String selectedContact = null;
    private String buyer;
    private List<String> messagesList;
    private int idpost;

    public MessagerieController(User user,App app){
        this.app=app;
        this.user=user;
    }

    public void initialize(){
        deco.setOnAction(event-> this.app.accueil());
        infoflo.setOnAction(event-> this.app.infoflorains(this.user));
        quitter.setOnAction(event->Platform.exit());
        profil.setOnAction(event->this.app.profil(this.user));
        tdb.setOnAction(event->this.app.tdb(this.user));
        offres.setOnAction(event->this.app.listoffres(this.user));
        florains.setText(this.user.getFlorains() +" florains");
        // Créer une liste de chaînes
        List<String> stringList = MessagerieController.getContactsOf(user);
        // Convertir la liste en ObservableList
        ObservableList<String> items = FXCollections.observableArrayList(stringList);
        // Remplir le ListView avec la liste observable
        lvContacts.setItems(items);

        lvContacts.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue != null) {
                this.buyer = newValue+"";
                // Charger les messages associés à l'utilisateur sélectionné
                this.messagesList = MessagerieController.messagesWith(user.getUsername(), this.buyer);
                ObservableList<String> messagesItems = FXCollections.observableArrayList(messagesList);
                lvMessages.setItems(messagesItems);
                
                // Mettre à jour la visibilité du TextArea en fonction de l'utilisateur sélectionné
                boolean isSystemUser = "System".equals(newValue); // Vérifier si l'utilisateur est "System"
                message.setVisible(!isSystemUser);
                envoyer.setVisible(!isSystemUser);
            }
        });

        lvMessages.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue != null) {
                if (newValue.matches("\\d+ ACCEPTER")) { // Vérifie si la chaîne est de la forme "int ACCEPTER"
                    
                    // Extraire l'entier de la chaîne
                    this.idpost = Integer.parseInt(newValue.split(" ")[0]);
                    if (!this.user.getUsername().equals(DatabaseManager.getPostbyId(this.idpost).getOwner().getUsername())) {

                    } else {
                        System.out.println(this.idpost);
                    
                        delMessage(this.buyer, this.user.getUsername());

                        DatabaseManager.acceptPost(this.user, this.buyer, this.idpost);
                        NotificationController.sendNotification(buyer, "Votre demande pour l'offre : "+DatabaseManager.getPostbyId(this.idpost).getTitle()+" a été acceptée.");

                        Alert alert = new Alert(Alert.AlertType.INFORMATION);
                        alert.setTitle("Information");
                        alert.setHeaderText("Offre acceptée !");
                        alert.setContentText("Vous avez accepté de partager votre offre avec "+this.buyer+"! Il en sera notifié et vous pourrez vous entendre sur les modalités via la messagerie.");
                        alert.show();
                    }

                    
                } else if (newValue.matches("\\d+ REFUSER")) {
                    this.idpost = Integer.parseInt(newValue.split(" ")[0]);
                    if (!this.user.getUsername().equals(DatabaseManager.getPostbyId(this.idpost).getOwner().getUsername())) {

                    } else {
                        delMessage(this.buyer, this.user.getUsername());
                        NotificationController.sendNotification(buyer, "Votre demande pour l'offre : "+DatabaseManager.getPostbyId(this.idpost).getTitle()+" a été refusée.");
                        Alert alert = new Alert(Alert.AlertType.INFORMATION);
                        alert.setTitle("Information");
                        alert.setHeaderText("Offre refusée !");
                        alert.setContentText("Vous avez refusé de partager votre offre avec "+this.buyer+"!");
                        alert.show();
                    }
                } else {
                    System.out.println();
                }
            }
        });

    
        envoyer.setOnAction(event -> {
            if (selectedContact != null && !message.getText().isEmpty()) {
                // Envoyer le message à l'utilisateur sélectionné
                MessagerieController.addMessage(user.getUsername(), selectedContact, message.getText());
    
                // Mettre à jour la liste des messages
                List<String> messagesList = MessagerieController.messagesWith(user.getUsername(), selectedContact);
                ObservableList<String> messagesItems = FXCollections.observableArrayList(messagesList);
                lvMessages.setItems(messagesItems);
    
                // Effacer le texte dans le TextField après l'envoi
                message.clear();
            }
        });

    }


    public static void addMessage(String usernameFROM, String usernameTO, String message) {
        try {
            Connection connection = DriverManager.getConnection(dbPath);
            PreparedStatement preparedStatement = connection.prepareStatement("INSERT INTO messagerie (idmsg, idconv, usernameFROM, usernameTO, message, time) VALUES (?, ?, ?, ?, ?, ?)");

            String existingIdConv = getIdConvIfExists(connection, usernameFROM, usernameTO);
            
            UUID uuid = UUID.randomUUID();
            String uniqueId = uuid.toString();
            preparedStatement.setString(1, uniqueId);
                    
            if (existingIdConv.equals("0")) {
                uuid = UUID.randomUUID();
                uniqueId = uuid.toString();
                preparedStatement.setString(2, uniqueId);
            } else {
                preparedStatement.setString(2, existingIdConv);
            }
            preparedStatement.setString(3, usernameFROM);
            preparedStatement.setString(4, usernameTO);
            preparedStatement.setString(5, message);

            int time = (int) System.currentTimeMillis() / 1000;
            preparedStatement.setInt(6, time);
            
            preparedStatement.executeUpdate();
            
            connection.close();
                    
        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Could not add message.");
        }
    }

    private static String getIdConvIfExists(Connection connection, String usernameFrom, String usernameTo) throws SQLException {
    String query = "SELECT idconv FROM messagerie WHERE (usernameFROM = ? AND usernameTO = ?) OR (usernameFROM = ? AND usernameTO = ?)";
    try (PreparedStatement preparedStatement = connection.prepareStatement(query)) {
        preparedStatement.setString(1, usernameFrom);
        preparedStatement.setString(2, usernameTo);
        preparedStatement.setString(3, usernameTo);
        preparedStatement.setString(4, usernameFrom);

        ResultSet resultSet = preparedStatement.executeQuery();
        if (resultSet.next()) {
            return resultSet.getString("idconv");
        }
        else {
            return "0";
        }
    }
}

    public static void delMessage(String usernameFROM, String usernameTO) {
        try {
            Connection connection = DriverManager.getConnection(dbPath);
            PreparedStatement preparedStatement = connection
                    .prepareStatement("DELETE FROM messagerie WHERE (usernameFROM = ? AND usernameTO = ?) OR (usernameFROM = ? AND usernameTO = ?);");
            preparedStatement.setString(1, usernameFROM);
            preparedStatement.setString(2, usernameTO);
            preparedStatement.setString(3, usernameTO);
            preparedStatement.setString(4, usernameFROM);
            preparedStatement.executeUpdate();
            connection.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    } 
    
    public static List<String> messagesWith(String usernameFROM, String usernameTO) {
        List<String> Messages = new ArrayList<>();
        String currentuser = usernameFROM;
        String message;
        try {
            Connection connection = DriverManager.getConnection(dbPath);
            String idconv = getIdConvIfExists(connection, usernameFROM, usernameTO);
            if (!"0".equals(idconv)) {
                String query = "SELECT message, usernameFROM FROM messagerie WHERE (idconv = ?) ORDER BY time";
                PreparedStatement preparedStatement = connection.prepareStatement(query);
                preparedStatement.setString(1, idconv);

                ResultSet resultSet = preparedStatement.executeQuery();
                    while (resultSet.next()) {
                        message = resultSet.getString("message");
                        if (currentuser.equals(resultSet.getString("usernameFROM"))) {
                            Messages.add("MOI");
                            Messages.add(message);
                        }
                        else {
                            Messages.add(usernameTO);
                            Messages.add(message);
                        }
                    }
                connection.close();
                return Messages;
            }
            else {
                connection.close();
                return Messages;
            }
            

        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Could not add user.");
            return Messages;
        }
    }

    public static List<String> getContactsOf(User username) {
        List<String> Contacts = new ArrayList<>();
        String currentuser = username.getUsername();
        try {
            Connection connection = DriverManager.getConnection(dbPath);
        
            String query = "SELECT DISTINCT usernameFROM, usernameTO FROM messagerie WHERE (usernameFROM = ?) OR (usernameTO = ?)";
            PreparedStatement preparedStatement = connection.prepareStatement(query);
            preparedStatement.setString(1, currentuser);
            preparedStatement.setString(2, currentuser);

            ResultSet resultSet = preparedStatement.executeQuery();
                while (resultSet.next()) {
                    
                    Contacts.add(resultSet.getString("usernameFROM"));
                    Contacts.add(resultSet.getString("usernameTO"));
                    
                }
            connection.close();
            return Contacts;

        } catch (Exception e) {
            e.printStackTrace();
            System.err.println("Could not add user.");
            return Contacts;
        }
    }

    public static void deleteMessagerieTable() {
        // SQLite connection string

        // SQL statement for creating a new table
        String sql = "DROP TABLE IF EXISTS messagerie;";

        try (Connection conn = DriverManager.getConnection(dbPath);
                Statement stmt = conn.createStatement()) {
            // create a new table
            stmt.execute(sql);
            conn.close();
        } catch (SQLException e) {
            System.out.println(e.getMessage());
        }
    }

}
