package telecomnancy.directdealing.controller;

import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.control.CheckMenuItem;
import javafx.scene.control.ListView;
import javafx.scene.control.Menu;
import javafx.scene.control.MenuButton;
import javafx.scene.control.MenuItem;
import telecomnancy.directdealing.App;
import telecomnancy.directdealing.database.DatabaseManager;
import telecomnancy.directdealing.model.Category;
import telecomnancy.directdealing.model.Post;
import telecomnancy.directdealing.model.User;

public class VoiroffresController {
    @FXML
    MenuItem tdb;
    @FXML
    MenuItem quitter;
    @FXML
    MenuItem deco;
    @FXML
    MenuItem profil;
    @FXML
    MenuItem messagerie;
    @FXML
    MenuItem offres;
    @FXML
    MenuItem infoflo;
    @FXML
    Menu florains;
    @FXML
    ListView<Post> lvOffres;

    @FXML
    CheckMenuItem filtreJardin;

    @FXML
    MenuButton filtresMenuButton;

    @FXML
    CheckMenuItem filtreCuisine;

    private ObservableList<Post> items;
    private List<Post> postsList;
    private List<CheckMenuItem> filters;
    private App app;
    private User user;

    public VoiroffresController(User user, App app) {
        this.user = user;
        this.app = app;
    }
    
    public void initialize() {
        this.postsList = DatabaseManager.getDbPosts();
        // AJOUTER LES FILTRES ICI !!
        deco.setOnAction(event -> this.app.accueil());
        quitter.setOnAction(event -> Platform.exit());
        profil.setOnAction(event -> this.app.profil(this.user));
        tdb.setOnAction(event -> this.app.tdb(this.user));
        messagerie.setOnAction(event -> this.app.messagerie(this.user));
        offres.setOnAction(event -> this.app.listoffres(this.user));
        infoflo.setOnAction(event->this.app.infoflorains(user));
        florains.setText(this.user.getFlorains() +" florains");
        filters = new LinkedList<>();
        for (Category category : Category.values()) {
            filters.add(new CheckMenuItem(category.toString()));
        }
        filtresMenuButton.getItems().addAll(filters);
        
        // Créer une liste de chaînes
        if (!(this.postsList==null)) {
            
        List<Integer> idposts = postsList.stream().map(p -> p.getId()).toList();
        // Convertir la liste en ObservableList
        items = FXCollections.observableList(postsList);
        
        // Remplir le ListView avec la liste observable
        lvOffres.setItems(items);
        
        filters.forEach(item -> item.setOnAction(event -> updateOffres()));
        
        lvOffres.setOnMouseClicked(event -> {
            // Vérifier si un élément est sélectionné
            if (lvOffres.getSelectionModel().getSelectedItem() != null) {
                int selectedIndex = lvOffres.getSelectionModel().getSelectedIndex();
                
                int currentIdpost = idposts.get(selectedIndex);
                
                // this.app.accueil();
                this.app.vueoffre(this.user, DatabaseManager.getPostbyId(currentIdpost));
            }
        });
    }
        
    }
    
    private void updateOffres() {
        items.clear();
        postsList = DatabaseManager.getDbPosts();
        boolean allFiltersUnchecked = true;
        for (CheckMenuItem filter : filters) {
            if (filter.isSelected()) {
                allFiltersUnchecked = false;
                List<Post> relevantPosts = postsList
                .stream()
                .filter(p -> p.getCategory().equalsIgnoreCase(filter.getText()))
                .toList();
                items.addAll(
                    relevantPosts);
                // System.out.println(postsList.size());
                lvOffres.setItems(items);
            }
        }
        if (allFiltersUnchecked) {
            items.addAll(postsList);
        }
        lvOffres.refresh();
    }
}
