package telecomnancy.directdealing.database;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Random;

import org.junit.jupiter.api.Test;

import telecomnancy.directdealing.controller.FlorainsController;
import telecomnancy.directdealing.model.Category;
import telecomnancy.directdealing.model.Frequency;
import telecomnancy.directdealing.model.Post;
import telecomnancy.directdealing.model.PostType;
import telecomnancy.directdealing.model.User;

public class DatabaseManagerTest {
    private User testUser;
    private User userA;
    private User userB;
    private Post testPost;
    private Post cuisinePost;
    private Post jardinPost;

    public DatabaseManagerTest() {
        DatabaseManager.intialize();
        testUser = new User(
                "Testeur",
                "testeur@mail.com",
                "testpwd",
                "060000");
        userA = new User(
            "User A",
            "userA@mail.com",
            "userApwd",
            "060000"
        );
        userB = new User(
            "User B",
            "userB@mail.com",
            "userBpwd",
            "060000"
        );
        DatabaseManager.addUser(testUser);
        DatabaseManager.addUser(userA);
        DatabaseManager.addUser(userB);
        int id = 64;
        String title = "test_post";
        String description = "test_description";
        String frequency = Frequency.PUNCTUAL.toString();
        PostType type = PostType.EQUIPMENT;
        int price = 100;
        long duration = 10;
        String startDate = LocalDateTime.now().toString();
        String endDate = LocalDateTime.now().toString();
        boolean available = true;
        String location = "Nancy";
        String category = Category.CUISINE.toString();
        testPost = new Post(id,
                testUser,
                title,
                description,
                frequency,
                type,
                price,
                duration,
                startDate,
                endDate,
                available,
                location,
                category);
        cuisinePost = new Post(id,
                testUser,
                title,
                description,
                frequency,
                type,
                price,
                duration,
                startDate,
                endDate,
                available,
                location,
                category);
        Random rand = new Random();
        cuisinePost.setId(rand.nextInt(2048));
        cuisinePost.setTitle("Cuisinier à domicile");
        cuisinePost.setCategory(Category.CUISINE.toString());
        jardinPost = new Post(id,
                testUser,
                title,
                description,
                frequency,
                type,
                price,
                duration,
                startDate,
                endDate,
                available,
                location,
                category);
        jardinPost.setId(rand.nextInt(2048));
        jardinPost.setTitle("Tondeuse");
        jardinPost.setCategory(Category.JARDINERIE.toString());
        DatabaseManager.addPost(cuisinePost);
        DatabaseManager.addPost(jardinPost);

    }

    @Test
    public void deleteUsersTableTest() {
        // DatabaseManager.deleteUsersTable();
    }

    @Test
    public void deleteDatabase() {
        // DatabaseManager.deleteDatabase();
    }

    @Test
    public void createDatabaseTest() {
        DatabaseManager.createDatabase();
        String path = "src/main/resources/DirectDealing.sqlite";
        File f = new File(path);
        assertTrue(f.exists());
    }

    // ---------------- GESTION DES UTILISATEURS -------------------------

    @Test
    public void createUsersTableTest() {
        DatabaseManager.createUsersTable();
        String sql = "SELECT name \n" +
                "FROM sqlite_master \n" +
                "WHERE type = 'table' AND name = 'users';";
        try {
            Connection connection = DriverManager.getConnection(DatabaseManager.getDbPath());
            Statement stmt = connection.createStatement();
            stmt.execute(sql);
            ResultSet result = stmt.getResultSet();
            assertTrue(result.next());
            assertEquals("users", result.getString("name"));
            connection.close();
        } catch (SQLException e) {
            assertTrue(false);
            e.printStackTrace();
        }

    }

    @Test
    public void testAddUser() {
        String username = "test_user";
        DatabaseManager.deleteUser(username);
        User testUser = new User(username, "null", "null", "null");
        DatabaseManager.addUser(testUser);
        String sql = "SELECT username \n" +
                "FROM users \n" +
                "WHERE username = '" + username + "';";
        try {
            Connection connection = DriverManager.getConnection(DatabaseManager.getDbPath());
            Statement stmt = connection.createStatement();
            stmt.execute(sql);
            ResultSet result = stmt.getResultSet();
            assertTrue(result.next());
            assertEquals(username, result.getString("username"));
            connection.close();
            DatabaseManager.deleteUser(username);
        } catch (SQLException e) {
            DatabaseManager.deleteUser(username);
            e.printStackTrace();
            assertTrue(false);
        }
    }

    @Test
    public void getUserByUsernameTest() {
        DatabaseManager.deleteUser(testUser);
        DatabaseManager.addUser(testUser);
        User result = DatabaseManager.getUserByUsername(testUser.getUsername());
        assertEquals(testUser.getUsername(), result.getUsername());
    }

    @Test
    public void checkCredentialsTest() {
        DatabaseManager.addUser(testUser);
        CredentialsCheck check = DatabaseManager.checkCredentials("Testeur", "testpwd");
        assertEquals(check, CredentialsCheck.CONNECTED);
    }

    // ------------------------ GESTION DES FLORAINS ---------------------------
    @Test
    public void setUserFlorainsTest() {
        DatabaseManager.deleteUser(testUser);
        DatabaseManager.addUser(testUser);
        int expectedAmount = 1234;
        DatabaseManager.setUserFlorains(testUser, expectedAmount);
        int newAmount = DatabaseManager.getUserFlorains(testUser.getUsername());
        assertEquals(newAmount, expectedAmount);
    }

    @Test
    public void addFlorainsTest() {
        DatabaseManager.deleteUser(testUser);
        DatabaseManager.addUser(testUser);
        int addAmount = 100;
        int expectedAmount = DatabaseManager.getUserFlorains(testUser) + addAmount;
        DatabaseManager.addFlorains(testUser.getUsername(), addAmount);
        int newAmount = DatabaseManager.getUserFlorains(testUser.getUsername());
        assertEquals(newAmount, expectedAmount);
    }

    @Test
    public void subFlorainsTest() {
        DatabaseManager.deleteUser(testUser);
        DatabaseManager.addUser(testUser);
        int subAmount = 100;
        int expectedAmount = DatabaseManager.getUserFlorains(testUser) + subAmount;
        DatabaseManager.addFlorains(testUser.getUsername(), subAmount);
        int newAmount = DatabaseManager.getUserFlorains(testUser.getUsername());
        assertEquals(newAmount, expectedAmount);
    }

    @Test
    public void transfertFlorainsTest(){
        int currentFlorainsA  = DatabaseManager.getUserFlorains(userA);
        int currentFlorainsB = DatabaseManager.getUserFlorains(userB);
        FlorainsController.transfertFlorains(userA.getUsername(),userB.getUsername(),50);
        int newFlorainsA  = DatabaseManager.getUserFlorains(userA);
        int newFlorainsB = DatabaseManager.getUserFlorains(userB);
        assertEquals(currentFlorainsA, newFlorainsA + 50);
        assertEquals(currentFlorainsB, newFlorainsB - 50);
    }
    // ------------------------ GESTION DES OFFRES ---------------------------

    @Test
    public void createPostTableTest() {
        DatabaseManager.createPostTable();
        String sql = "SELECT name \n" +
                "FROM sqlite_master \n" +
                "WHERE type = 'table' AND name = 'Posts';";
        try {
            Connection connection = DriverManager.getConnection(DatabaseManager.getDbPath());
            Statement stmt = connection.createStatement();
            stmt.execute(sql);
            ResultSet result = stmt.getResultSet();
            assertTrue(result.next());
            assertEquals("Posts", result.getString("name"));
            connection.close();
        } catch (SQLException e) {
            assertTrue(false);
            e.printStackTrace();
        }
    }

    @Test
    public void addPostTest() {
        DatabaseManager.deletePost(testPost);
        DatabaseManager.addPost(testPost);
        String sql = "SELECT title \n" +
                "FROM Posts \n" +
                "WHERE title = '" + testPost.getTitle() + "';";
        try {
            Connection connection = DriverManager.getConnection(DatabaseManager.getDbPath());
            Statement stmt = connection.createStatement();
            stmt.execute(sql);
            ResultSet result = stmt.getResultSet();
            assertTrue(result.next());
            assertEquals(testPost.getTitle(), result.getString("title"));
            connection.close();
            DatabaseManager.deleteUser(testPost.getTitle());
        } catch (SQLException e) {
            DatabaseManager.deleteUser(testPost.getTitle());
            e.printStackTrace();
            assertTrue(false);
        }
    }

    @Test
    public void getDbPostsTest() {
        DatabaseManager.addPost(testPost);
        List<Post> posts = DatabaseManager.getDbPosts();
        assertTrue(posts
                .stream()
                .map(p -> p.getTitle())
                .toList()
                .contains(testPost.getTitle()));
    }

    @Test
    public void createDemandQueueTableTest() {
        DatabaseManager.createDemandQueueTable();
        String sql = "SELECT name \n" +
                "FROM sqlite_master \n" +
                "WHERE type = 'table' AND name LIKE 'DemandQueues';";
        try {
            Connection connection = DriverManager.getConnection(DatabaseManager.getDbPath());
            Statement stmt = connection.createStatement();
            stmt.execute(sql);
            ResultSet result = stmt.getResultSet();
            assertTrue(result.next());
            assertEquals("DemandQueues", result.getString("name"));
            connection.close();
        } catch (SQLException e) {
            assertTrue(false);
            e.printStackTrace();
        }
    }

    // @Test
    // public void getDbPostsByCategoryTest(){
    // DatabaseManager.getDbPostsByCategory(null)
    // }

}
